"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _path = _interopRequireDefault(require("path"));

var _Cache = _interopRequireDefault(require("./Cache"));

var _defaults = _interopRequireDefault(require("./defaults"));

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
const CACHE_SIZES = {
  idx: 8 * 1024,
  key: 6 * 1024,
  val: 4 * 1024
};

class Files {
  constructor(base, file, options) {
    this._isTrie = false;
    this._isOpen = false;
    this._fds = [];
    this._path = void 0;
    this._isTrie = options.isTrie;
    this._path = _path.default.join(base, file);

    _mkdirp.default.sync(this._path);
  }

  close() {
    this._fds.forEach(fds => Object.values(fds).forEach((_ref) => {
      let {
        fd
      } = _ref;
      return _fs.default.closeSync(fd);
    }));

    this._isOpen = false;
  }

  open() {
    for (let i = 0; i < _defaults.default.HDR_SPLIT_FILES; i++) {
      this._fds[i] = ['idx', 'key', 'val'].reduce((fds, type) => {
        const prefix = "0".concat(i.toString(16)).slice(-2);

        const file = _path.default.join(this._path, "".concat(_defaults.default.VERSION, ".").concat(prefix, ".").concat(type));

        if (!_fs.default.existsSync(file)) {
          _fs.default.writeFileSync(file, new Uint8Array(type === 'idx' ? _defaults.default.HDR_TOTAL_SIZE : type === 'key' ? _defaults.default.KEY_TOTAL_SIZE : 0));
        }

        const cache = new _Cache.default(CACHE_SIZES[type]);

        const fd = _fs.default.openSync(file, 'r+');

        const size = _fs.default.fstatSync(fd).size;

        fds[type] = {
          cache,
          fd,
          file,
          size
        };
        return fds;
      }, {});
    }
  }

  __append(type, index, buffer) {
    const fd = this._fds[index][type];
    const offset = fd.size;

    _fs.default.writeSync(fd.fd, buffer, 0, buffer.length, offset);

    fd.size += buffer.length;
    fd.cache.set(offset, buffer);
    return offset;
  }

  __read(type, index, offset, length) {
    const fd = this._fds[index][type];
    const cached = fd.cache.get(offset);

    if (cached) {
      return cached;
    }

    const buffer = new Uint8Array(length);

    _fs.default.readSync(fd.fd, buffer, 0, length, offset);

    fd.cache.set(offset, buffer);
    return buffer;
  }

  __update(type, index, offset, buffer) {
    const fd = this._fds[index][type];

    _fs.default.writeSync(fd.fd, buffer, 0, buffer.length, offset);

    fd.cache.set(offset, buffer);
    return offset;
  }

  _appendHdr(index, buffer) {
    return this.__append('idx', index, buffer) / _defaults.default.HDR_TOTAL_SIZE;
  }

  _appendKey(index, buffer) {
    return this.__append('key', index, buffer) / _defaults.default.KEY_TOTAL_SIZE;
  }

  _appendVal(index, buffer) {
    return this.__append('val', index, buffer);
  }

  _readHdr(index, offset) {
    return this.__read('idx', index, offset * _defaults.default.HDR_TOTAL_SIZE, _defaults.default.HDR_TOTAL_SIZE);
  }

  _readKey(index, offset) {
    return this.__read('key', index, offset * _defaults.default.KEY_TOTAL_SIZE, _defaults.default.KEY_TOTAL_SIZE);
  }

  _readVal(index, offset, length) {
    return this.__read('val', index, offset, length);
  }

  _updateHdr(index, offset, buffer) {
    return this.__update('idx', index, offset * _defaults.default.HDR_TOTAL_SIZE, buffer) / _defaults.default.HDR_ENTRY_SIZE;
  }

  _updateKey(index, offset, buffer) {
    return this.__update('key', index, offset * _defaults.default.KEY_TOTAL_SIZE, buffer) / _defaults.default.KEY_TOTAL_SIZE;
  }

}

exports.default = Files;