"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.readU8aU32 = readU8aU32;
exports.writeU8aU32 = writeU8aU32;
exports.modifyHdr = modifyHdr;
exports.modifyKey = modifyKey;
exports.newHdr = newHdr;
exports.newKey = newKey;
exports.parseHdr = parseHdr;
exports.parseKey = parseKey;
exports.serializeKey = serializeKey;

var _utilCrypto = require("@polkadot/util-crypto");

var _defaults = _interopRequireDefault(require("./defaults"));

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
function readU8aU32(u8a, offset) {
  return u8a[offset] + (u8a[offset + 1] << 8) + (u8a[offset + 2] << 16) + (u8a[offset + 3] << 24);
}

function writeU8aU32(u8a, value, offset) {
  u8a[offset] = value & 0xff;
  u8a[offset + 1] = value >> 8 & 0xff;
  u8a[offset + 2] = value >> 16 & 0xff;
  u8a[offset + 3] = value >> 24 & 0xff;
}

function modifyHdr(hdr, hdrIndex, type, at) {
  writeU8aU32(hdr, at << 2 | type, hdrIndex * _defaults.default.HDR_ENTRY_SIZE);
  return hdr;
}

function modifyKey(keyData, valAt, valSize) {
  writeU8aU32(keyData, valAt, _defaults.default.KEY_DATA_SIZE);
  writeU8aU32(keyData, valSize, _defaults.default.KEY_DATA_SIZE + _defaults.default.U32_SIZE);
  return keyData;
}

function newHdr(indexes) {
  const hdr = new Uint8Array(_defaults.default.HDR_TOTAL_SIZE);
  indexes.forEach((_ref) => {
    let {
      dataAt,
      hdrIndex,
      type
    } = _ref;
    return modifyHdr(hdr, hdrIndex, type, dataAt);
  });
  return hdr;
}

function newKey(key, _ref2) {
  let {
    valAt,
    valSize
  } = _ref2;
  const keyData = new Uint8Array(_defaults.default.KEY_TOTAL_SIZE);
  keyData.set(key.buffer, 0);
  return modifyKey(keyData, valAt, valSize);
}

function parseHdr(hdr, hdrIndex) {
  const value = readU8aU32(hdr, hdrIndex * _defaults.default.HDR_ENTRY_SIZE);
  return {
    at: value >> 2,
    type: value & 0b11
  };
}

function parseKey(keyData) {
  return {
    valAt: readU8aU32(keyData, _defaults.default.KEY_DATA_SIZE),
    valSize: readU8aU32(keyData, _defaults.default.KEY_DATA_SIZE + _defaults.default.U32_SIZE)
  };
}

function serializeKey(u8a) {
  // Convert any non-32-byte keys into a hash of the key. This allows for proper
  // key distribution. In practice, the inputs should already be hashed, in the
  // case of using a trie, however if used directly, this would come into play
  const buffer = u8a.length === _defaults.default.KEY_DATA_SIZE ? u8a : (0, _utilCrypto.blake2AsU8a)(u8a); // for 16 files, with 16 entries per header

  const parts = new Uint8Array(_defaults.default.KEY_PARTS_SIZE);
  const index = 0; // buffer[0] & 0b1111;

  let count = 0; // 1;
  // parts[0] = buffer[0] >> 4;

  for (let i = 0; i < _defaults.default.KEY_DATA_SIZE; i++) {
    const item = buffer[i];
    parts[count] = item & 0b1111;
    parts[count + 1] = item >> 4;
    count += 2;
  }

  return {
    buffer,
    index,
    parts
  };
}